/*
 * ANTs - Fourmis informatiques
 * Copyright (C) 2000 MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#define __NEST_C

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <math.h>

#include <assert.h>

#include "nest.h"

/* Concret data type */
typedef struct elem {
  int object;  /* object */
} elem;
/* The nest */
typedef struct Nest {
  elem **nest;  /* nest */
  int xmax;     /* width */
  int ymax;     /* height */
} _Nest;


/**
 * Constructor.
 */
Nest nest_new(int x, int y) {
  Nest n = (Nest)malloc(sizeof(_Nest));
  if(!n) { fprintf(stderr, "Can't allocate memory (Nest)\n"); exit(0); }

  /* Init memory with 0 */
  memset(n, 0, sizeof(_Nest));

  /* Allocs */
  n->xmax = x;
  n->ymax = y;
  n->nest = (elem **)malloc(sizeof(elem*) * y);
  for(int i=0; i<y; i++)
    n->nest[i] = (elem *)malloc(sizeof(elem) * x);
  
  /* Inits */
  for(int i=0; i<y; i++)
    for(int j=0; j<x; j++) {
      float p = (float)rand()/(float)RAND_MAX;
      if(p <= 3.0/4.0)
        n->nest[i][j].object = NONE;
      else if(p > 3.0/4.0 && p <= 3.0/4.0+1.0/12.0)
        n->nest[i][j].object = EGG;
      else if(p > 3.0/4.0+1.0/12.0 && p <= 3.0/4.0+2.0/12.0)
        n->nest[i][j].object = LARVA;
      else if(p > 3.0/4.0+2.0/12.0)
        n->nest[i][j].object = COCOON;
      else
        n->nest[i][j].object = NONE;
    }
  
  return n;
}

/*
 * Destructor.
 */
int nest_destroy(Nest n) {
  /* Assertion */
  assert(n && n->nest);

  /* Free memory */
  for(int i=0; i<n->ymax; i++)
    free(n->nest[i]);
  free(n->nest);
  free(n);

  return 1;
}


/**
 * SizeX (begin at 0).
 */
int nest_xmax(Nest n) {
  /* Assertion */
  assert(n && n->nest);
  
  return n->xmax;
}

/**
 * SizeY (begin at 0).
 */
int nest_ymax(Nest n) {
  /* Assertion */
  assert(n && n->nest);

  return n->ymax;
}


/**
 * Object encounterred?
 * Or, pos (x,y) is busy?
 */
int nest_is_busy(Nest n, int x, int y) {
  /* Assertion */
  assert(n && n->nest);
  assert(x >= 0); assert(x < n->xmax);
  assert(y >= 0); assert(y < n->ymax);

  return n->nest[y][x].object;
}

/**
 * Update nest (object) in pos (x, y).
 */
int nest_put_down(Nest n, int x, int y, int o) {
  /* Assertion */
  assert(n && n->nest);
  assert(x >= 0); assert(x < n->xmax);
  assert(y >= 0); assert(y < n->ymax);

  return (n->nest[y][x].object = o);
}

