/* 
 * Stack abstract data type - Last In First Out.
 * Copyright (C) 2003 MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#define __STACK_C

#include <stdio.h>
#include <stdlib.h>
#include <string.h>

#include <assert.h>

#include "stack.h"

/* Concret data type */
typedef struct Stack {
  double *tab;  /* elements (double) */
  int size;     /* tab. size */
  int top;      /* head position */
} _Stack;


#define STEP 128 /* incr. for realloc */


/**
 * Constructor. 
 */
Stack stack_new(void) {
  /* Alloc _Stack struct */
  Stack stack = malloc(sizeof(_Stack));
  if(!stack) { fprintf(stderr, "Can't allocate memory (Stack)\n"); exit(0); }

  /* Init struct */
  stack->top = 0;
  stack->size = STEP;

  /* Init memory */
  memset(stack, 0, sizeof(_Stack));

  /* Alloc tab. of elements */
  stack->tab = malloc(sizeof(double) * STEP);
  if(!stack) { fprintf(stderr, "Can't allocate memory (Stack->tab)\n"); exit(0); }

  return stack;
}

/**
 * Destructor.
 */
int stack_destroy(Stack stack) {
  /* Assertion */
  assert(stack && stack->tab);

  /* Free memory */
  free(stack->tab);
  free(stack);

  return 1;
}


/**
 * Push n.
 * With memory reallocation if necessary.
 */
int stack_push(Stack stack, double n) {
  /* Assertion */
  assert(stack && stack->tab);
  
  if(stack->top >= stack->size) {
    /* Realloc, stack too small */
    stack->tab = realloc(stack->tab, sizeof(double) * (stack->size + STEP));
    if(!stack) { fprintf(stderr, "Can't reallocate memory (Stack->tab)\n"); return 0; }
    stack->size += STEP;
  }
  /* Add the new element upon the stack */
  stack->tab[stack->top++] = n;

  return stack->tab[stack->top-1];
}

/**
 * Pop.
 * Test if stack is empty?
 */
double stack_pop(Stack stack) {
  /* Assertion */
  assert(stack && stack->tab);

  if(stack->top > 0) {
    return stack->tab[--stack->top];
  }
  else {
    fprintf(stderr, "Empty stack\n");
    return 0;
  }
}


/**
 * Top.
 * Test if stack is empty?
 */
double stack_top(Stack stack) {
  /* Assertion */
  assert(stack && stack->tab);

  /* Get top */
  int pos = stack->top;
  return stack->tab[--pos];
}

/**
 * Head idx.
 */
int stack_top_index(Stack stack) {
  /* Assertion */
  assert(stack && stack->tab);

  /* Get head index */
  return stack->top;
}


/**
 * Stack is full? 0|1
 */
int stack_is_full(Stack stack) {
  /* Assertion */
  assert(stack && stack->tab);

  /* Full? */
  return stack->top >= stack->size;
}

/**
 * Stack is empty? 0|1
 */
int stack_is_empty(Stack stack) {
  /* Assertion */
  assert(stack && stack->tab);

  /* Empty? */
  return stack->top <= 0;
}


/**
 * Get elem (pos=i). 
 */
double stack_elem(Stack stack, int i) {
  /* Assertion */
  assert(stack && stack->tab);

  /* Get element[i] */
  if(0 <= i && i < stack->top) {
    return stack->tab[i];
  }
  else {
    fprintf(stderr, "Bad index\n"); 
    return 0;
  }
}
