/*
 * ANTs - Fourmis informatiques
 * Copyright (C) 2000 MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#include <stdio.h>
#include <stdlib.h>
#include <math.h>

#include <assert.h>

#include "nest.h"
#include "field.h"
#include "ant.h"

#define MAX(a, b) ((a)>(b))?(a):(b)

#define SQ(x) ((x)*(x))
#define K1 1.0
#define K2 1.0

/* Probabilities vs/ occupation rate */
//#define P1(a) (SQ(K1/(K1+ant_occupation_rate(a))))
//#define P2(a) (SQ(ant_occupation_rate(a)/(K2+ant_occupation_rate(a))))
//#define P1(a) (SQ(1-ant_occupation_rate(a)))
//#define P2(a) (SQ(ant_occupation_rate(a)))
#define P1(a) (pow(1-ant_occupation_rate(a), 3))
#define P2(a) (pow(ant_occupation_rate(a), 3))
#define P3(a, n) (pow(1-ant_occupation_rate2(a, n), 3))
#define P4(a, n) (pow(ant_occupation_rate2(a, n), 3))

/* Directivity of movements */
//#define D0() (5.0/24.0)  /* little directive */
//#define D1() (8.0/24.0)
//#define D2() (6.0/24.0)
//#define D3() (4.0/24.0)
//#define D4() (1.0/24.0)
//#define D0() (12.0/19.0)  /* moderately directive */
//#define D1() (4.0/19.0)
//#define D2() (2.0/19.0)
//#define D3() (1.0/19.0)
//#define D4() (0.0)
#define D0() (12.0/16.0)  /* very directive */
#define D1() (2.0/16.0)
#define D2() (1.0/16.0)
#define D3() (1.0/16.0)
#define D4() (0.0)


/**
 * Compute the occupation rate according to neighbourhood.
 */
float ant_occupation_rate2(Ant a, Nest n) {
  /* Assertion */
  assert(a && n);
  int i = ant_posx(a);
  int j = ant_posy(a);
  assert(i >= 0); assert(i < nest_xmax(n));
  assert(j >= 0); assert(j < nest_ymax(n));

  /* Update object */
  int object;
  if(ant_is_loaded(a)) object = ant_is_loaded(a);
  else if(nest_is_busy(n, i, j)) object = nest_is_busy(n, i, j);
  else object = 0;

  /* Compute occupation rate */
  int occup=0, max=0;
  if(i+1 < nest_xmax(n)) { 
    if(object == nest_is_busy(n, i+1, j)) occup++;
    max++; 
  }
  if(i+1 < nest_xmax(n) && j+1 < nest_ymax(n)) {
    if(object == nest_is_busy(n, i+1, j+1)) occup++; 
    max++; 
  }
  if(j+1 < nest_ymax(n)) {
    if(object == nest_is_busy(n, i, j+1)) occup++; 
    max++; 
  }
  if(i-1 >= 0 && j+1 < nest_ymax(n)) {
    if(object == nest_is_busy(n, i-1, j+1)) occup++; 
    max++; 
  }
  if(i-1 >= 0) {      
    if(object == nest_is_busy(n, i-1, j)) occup++; 
    max++; 
  }
  if(i-1 >= 0 && j-1 >= 0) {
    if(object == nest_is_busy(n, i-1, j-1)) occup++; 
    max++; 
  }
  if(j-1 >= 0) {
    if(object == nest_is_busy(n, i, j-1)) occup++; 
    max++; 
  }
  if(i+1 < nest_xmax(n) && j-1 >= 0) {
    if(object == nest_is_busy(n, i+1, j-1)) occup++; 
    max++; 
  }

  if(i+2 < nest_xmax(n) && j-1 >= 0) {   
    if(object == nest_is_busy(n, i+2, j-1)) occup++; 
    max++; 
  }
  if(i+2 < nest_xmax(n)) {
    if(object == nest_is_busy(n, i+2, j)) occup++; 
    max++; 
  }
  if(i+2 < nest_xmax(n) && j+1 < nest_ymax(n)) {
    if(object == nest_is_busy(n, i+2, j+1)) occup++; 
    max++; 
  }

  if(i+1 < nest_xmax(n) && j+2 < nest_ymax(n)) {
    if(object == nest_is_busy(n, i+1, j+2)) occup++; 
    max++; 
  }
  if(j+2 < nest_ymax(n)) {
    if(object == nest_is_busy(n, i, j+2)) occup++; 
    max++; 
  }
  if(i-1 >= 0 && j+2 < nest_ymax(n)) { 
    if(object == nest_is_busy(n, i-1, j+2)) occup++; 
    max++; 
  }

  if(i-2 >= 0 && j+1 < nest_ymax(n)) {
    if(object == nest_is_busy(n, i-2, j+1)) occup++; 
    max++; 
  }
  if(i-2 >= 0) {
    if(object == nest_is_busy(n, i-2, j)) occup++; 
    max++; 
  }
  if(i-2 >= 0 && j-1 >= 0) {
    if(object == nest_is_busy(n, i-2, j-1)) occup++; 
    max++; 
  }

  if(i-1 >= 0 && j-2 >= 0) {
    if(object == nest_is_busy(n, i-1, j-2)) occup++; 
    max++; 
  }
  if(j-2 >= 0) {
    if(object == nest_is_busy(n, i, j-2)) occup++; 
    max++; 
  }
  if(i+1 < nest_xmax(n) && j-2 >= 0) {
    if(object == nest_is_busy(n, i+1, j-2)) occup++; 
    max++; 
  }

  return (float)occup/(float)max;
}

/**
 * Can I load?
 */
short ant_can_load(Ant a, Nest n) {
  /* Assertion */
  assert(a && n);

  if(!ant_is_loaded(a) && nest_is_busy(n, ant_posx(a), ant_posy(a))) {
    if(((float)rand() / (float)RAND_MAX) < P1(a)) {
      ant_load(a, nest_is_busy(n, ant_posx(a), ant_posy(a)));
      nest_put_down(n, ant_posx(a), ant_posy(a), NONE);
      return 1;
    }
    else return 0;
  }
  else return 0;
}

short ant_can_load2(Ant a, Nest n) {
  /* Assertion */
  assert(a && n);

  if(!ant_is_loaded(a) && nest_is_busy(n, ant_posx(a), ant_posy(a))) {
    if(((float)rand() / (float)RAND_MAX) < P3(a, n)) {
      ant_load(a, nest_is_busy(n, ant_posx(a), ant_posy(a)));
      nest_put_down(n, ant_posx(a), ant_posy(a), NONE);
      return 1;
    }
    else return 0;
  }
  else return 0;
}

/**
 * Can I unload?
 */
short ant_can_unload(Ant a, Nest n) {
  /* Assertion */
  assert(a && n);

  if(!nest_is_busy(n, ant_posx(a), ant_posy(a)) && ant_is_loaded(a)) {
    if(((float)rand() / (float)RAND_MAX) < P2(a)) {
      nest_put_down(n, ant_posx(a), ant_posy(a), ant_unload(a));
      return 1;
    }
    else return 0;
  }
  else return 0;
}

short ant_can_unload2(Ant a, Nest n) {
  /* Assertion */
  assert(a && n);

  if(!nest_is_busy(n, ant_posx(a), ant_posy(a)) && ant_is_loaded(a)) {
    if(((float)rand() / (float)RAND_MAX) < P4(a, n)) {
      nest_put_down(n, ant_posx(a), ant_posy(a), ant_unload(a));
      return 1;
    }
    else return 0;
  }
  else return 0;
}

/**
 * Move the ant on its nest, then load/unload objects.
 */
int ant_move(Ant a, Nest n, int d) {
  /* Assertion */
  assert(a && n);
  assert(ant_posx(a) >= 0); assert(ant_posx(a) < nest_xmax(n));
  assert(ant_posy(a) >= 0); assert(ant_posy(a) < nest_ymax(n));

  /* Displacement */
  switch(d) {
  case NORTH:
    ant_movey(a, -1);
    if(ant_posy(a) < 0) ant_movey(a, 2);
    break;
  case EAST:
    ant_movex(a, 1);
    if(ant_posx(a) >= nest_xmax(n)) ant_movex(a, -2);
    break;
  case SOUTH:
    ant_movey(a, 1);
    if(ant_posy(a) >= nest_ymax(n)) ant_movey(a, -2);
    break;
  case WEST:
    ant_movex(a, -1);
    if(ant_posx(a) < 0) ant_movex(a, 2);
    break;
  case NORTH_EAST:
    ant_movey(a, -1);
    if(ant_posy(a) < 0) ant_movey(a, 2);
    ant_movex(a, 1);
    if(ant_posx(a) >= nest_xmax(n)) ant_movex(a, -2);
    break;
  case SOUTH_EAST:
    ant_movey(a, 1);
    if(ant_posy(a) >= nest_ymax(n)) ant_movey(a, -2);
    ant_movex(a, 1);
    if(ant_posx(a) >= nest_xmax(n)) ant_movex(a, -2);
    break;
  case SOUTH_WEST:
    ant_movey(a, 1);
    if(ant_posy(a) >= nest_ymax(n)) ant_movey(a, -2);
    ant_movex(a, -1);
    if(ant_posx(a) < 0) ant_movex(a, 2);
    break;
  case NORTH_WEST:
    ant_movey(a, -1);
    if(ant_posy(a) < 0) ant_movey(a, 2);
    ant_movex(a, -1);
    if(ant_posx(a) < 0) ant_movex(a, 2);
    break;
  default:
    break;
  }

  /* Update memory */
  ant_add_memory(a, nest_is_busy(n, ant_posx(a), ant_posy(a)));
  /* Work */
  ant_can_load(a, n);
  ant_can_unload(a, n); 

  return 1;
}


/**
 * Can I collect?
 */
short ant_can_collect(Ant a, Field f) {
  /* Assertion */
  assert(a && f);

  if(!ant_is_loaded(a) && field_is_busy(f, ant_posx(a), ant_posy(a))) {
    if(1) {  /* always collect */
      ant_load(a, field_is_busy(f, ant_posx(a), ant_posy(a)));
      field_put_down(f, ant_posx(a), ant_posy(a), NONE);
      return 1;
    }
    else return 0;
  }
  else return 0;
}

/**
 * Can I deposit?
 */
short ant_can_deposit(Ant a, Field f) {
  /* Assertion */
  assert(a && f);

  if(ant_is_loaded(a)) {
#if 0  /* accumulate in top-lift corner */
    if(ant_posx(a) < field_xmax(f)/10 && ant_posy(a) < field_ymax(f)/10) {
      if(!field_is_busy(f, ant_posx(a), ant_posy(a))) {
        field_put_down(f, ant_posx(a), ant_posy(a), ant_unload(a));
        return 1;
      }
      else if(ant_posx(a) == 0 && ant_posy(a) == 0) {  /* force @ (0, 0) */
        field_put_down(f, ant_posx(a), ant_posy(a), ant_unload(a));
        return 1;
      }
      else return 0;
    }
#else  /* nest @ (0, 0) */
    if(ant_posx(a) == 0 && ant_posy(a) == 0) {
      ant_unload(a);
      return 1;
    }
#endif
    else return 0;
  }
  else return 0;
}

/**
 * Put pheromone on the track followed by the ant.
 */
int ant_put_down_pheromone(Ant a, Field f) {
  /* Assertion */
  assert(a && f);

  field_incr_pheromone(f, ant_posx(a), ant_posy(a));

  return 1;
}

/**
 * Choose the best cap to follow.
 */
static int ant_best_direction(Ant a, Field f) {
  /* Assertion */
  assert(a && f);
  assert(ant_posx(a) >= 0); assert(ant_posx(a) < field_xmax(f));
  assert(ant_posy(a) >= 0); assert(ant_posy(a) < field_ymax(f));
  
  int best_dir = SOUTH_EAST;  /* moving away */

  /* Search the best cap */
  if(ant_is_loaded(a)) {  /* return (top-left) */
    best_dir = NORTH_WEST;      

    float dx = ant_posx(a) - 0;
    float dy = ant_posy(a) - 0;
    if(dx > 2 * dy)
      best_dir = WEST;
    else if(dy > 2 * dx)
      best_dir = NORTH;
    //printf("ant(%2d, %2d) best_dir=%d (return)\n", ant_posx(a), ant_posy(a), best_dir);
  }
  else {  /* research (south-east) */
#if PHEROMONE
    double dir[8]; /* pheromone around the ant */

    if(ant_posy(a)-1 < 0) 
      dir[NORTH] = 0.0;
    else 
      dir[NORTH] = field_taux_pheromone(f, ant_posx(a), ant_posy(a)-1);;
      
    if(ant_posy(a)-1 < 0 || ant_posx(a)+1 >= field_xmax(f))
      dir[NORTH_EAST] = 0.0;
    else
      dir[NORTH_EAST] = field_taux_pheromone(f, ant_posx(a)+1, ant_posy(a)-1);;
    
    if(ant_posx(a)+1 >= field_xmax(f))
      dir[EAST] = 0.0;
    else
      dir[EAST] = field_taux_pheromone(f, ant_posx(a)+1, ant_posy(a));;
    
    if(ant_posy(a)+1 >= field_ymax(f) || ant_posx(a)+1 >= field_xmax(f))
      dir[SOUTH_EAST] = 0.0;
    else
      dir[SOUTH_EAST] = field_taux_pheromone(f, ant_posx(a)+1, ant_posy(a)+1);;
    
    if(ant_posy(a)+1 >= field_ymax(f))
      dir[SOUTH] = 0.0;
    else
      dir[SOUTH] = field_taux_pheromone(f, ant_posx(a), ant_posy(a)+1);;
    
    if(ant_posy(a)+1 >= field_ymax(f) || ant_posx(a)-1 < 0)
      dir[SOUTH_WEST] = 0.0;
    else
      dir[SOUTH_WEST] = field_taux_pheromone(f, ant_posx(a)-1, ant_posy(a)+1);;
    
    if(ant_posx(a)-1 < 0)
      dir[WEST] = 0.0;
    else
      dir[WEST] = field_taux_pheromone(f, ant_posx(a)-1, ant_posy(a));;
    
    if(ant_posy(a)-1 < 0 || ant_posx(a)-1 < 0)
      dir[NORTH_WEST] = 0.0;
    else
      dir[NORTH_WEST] = field_taux_pheromone(f, ant_posx(a)-1, ant_posy(a)-1);;

    /* Select best direcion:
     * | 7 | 0 | 1 |
     * | 6 | A | 2 |
     * | 5 | 4 | 3 |
     */
    for(int i=1; i<=4; i++) {  /* moving away */
      if(dir[i] > 0.0) {  /* pheromone */
        if(dir[i] > dir[best_dir]) best_dir = i;
        else if(dir[i] == dir[best_dir]) best_dir = ((float)rand() < (float)RAND_MAX/2)? i : best_dir;
      }
    }
    if(dir[best_dir] == 0.0)  /* no pheromone */
#else
    if (1)
#endif
    {
#if DIRECTIVITY
      /* Directivity probas:
       * | D4 | D3 | D2 |
       * | D3 | SE | D1 |
       * | D2 | D1 | D0 |
       */
      double p = (float)rand() / (float)RAND_MAX;

      if(p < D0())  /* FORWARD */
        best_dir = ant_direction(a);
      else if(p < D0() + D1()/2.0)  /* FORWARD_RIGHT */
        best_dir = (ant_direction(a) + 1) % 8;
      else if(p < D0() + D1())  /* FORWARD_LEFT */
        best_dir = (ant_direction(a) + 7) % 8;
      else if(p < D0() + D1() + D2()/2.0)  /* RIGHT */
        best_dir = (ant_direction(a) + 2) % 8;
      else if(p < D0() + D1() + D2())  /* LEFT */
        best_dir = (ant_direction(a) + 6) % 8;
      else if(p < D0() + D1() + D2() + D3()/2.0)  /* BACKWARD_RIGHT */
        best_dir = (ant_direction(a) + 3) % 8;
      else if(p < D0() + D1() + D2() + D3())  /* BACKWARD_LEFT */
        best_dir = (ant_direction(a) + 5) % 8;
      else if(p < D0() + D1() + D2() + D3() + D4())  /* BACKWARD */
        best_dir = (ant_direction(a) + 4) % 8;
#else
      /* Random movement */
      best_dir = (int)(((float)rand()/(float)RAND_MAX) * 8.0);
#endif
    }
    //printf("ant(%2d, %2d) best_dir=%d (research)\n", ant_posx(a), ant_posy(a), best_dir);
  } /* end research */
  
  /* Correct direction vs/ border */
  switch(best_dir) {
  case NORTH:
    if(ant_posy(a)-1 < 0) {
      if(ant_posx(a)+1 >= field_xmax(f))
        best_dir = WEST;
      else
        best_dir = EAST;
    }
    break;
  case NORTH_EAST:
    if(ant_posy(a)-1 < 0 && ant_posx(a)+1 >= field_xmax(f))
      best_dir = SOUTH;
    else if(ant_posy(a)-1 < 0)
      best_dir = EAST;
    else if(ant_posx(a)+1 >= field_xmax(f))
      best_dir = NORTH;
    break;
  case EAST:
    if(ant_posx(a)+1 >= field_xmax(f)) {
      if(ant_posy(a)+1 >= field_ymax(f))
        best_dir = NORTH;
      else
        best_dir = SOUTH;          
    }
    break;
  case SOUTH_EAST:
    if(ant_posy(a)+1 >= field_ymax(f) && ant_posx(a)+1 >= field_xmax(f))
      best_dir = NORTH_WEST;
    else if(ant_posy(a)+1 >= field_ymax(f))
      best_dir = EAST;
    else if(ant_posx(a)+1 >= field_xmax(f))
      best_dir = SOUTH;
    break;
  case SOUTH:
    if(ant_posy(a)+1 >= field_ymax(f)) {
      if(ant_posx(a)+1 >= field_xmax(f))
        best_dir = WEST;
      else
        best_dir = EAST;
    }
    break;
  case SOUTH_WEST:
    if(ant_posy(a)+1 >= field_ymax(f) && ant_posx(a)-1 < 0)
      best_dir = EAST;
    else if(ant_posy(a)+1 >= field_ymax(f))
      best_dir = WEST;
    else if(ant_posx(a)-1 < 0)
      best_dir = SOUTH;
    break;
  case WEST:
    if(ant_posx(a)-1 < 0) {
      if(ant_posy(a)+1 >= field_ymax(f))
        best_dir = NORTH;
      else
        best_dir = SOUTH;
    }
    break;
  case NORTH_WEST:
    if(ant_posy(a)-1 < 0 && ant_posx(a)-1 < 0)
      best_dir = SOUTH_EAST;
    else if(ant_posy(a)-1 < 0)
      best_dir = WEST;
    else if(ant_posx(a)-1 < 0)
      best_dir = NORTH;
    break;
  default:
    break;
  }
  //printf("            best_dir=%d\n", best_dir);

  return best_dir;
}

/**
 * Move the ant according to pheromones, then load/unload objects.
 */
int ant_move_following_pheromone(Ant a, Field f) {
  /* Assertion */
  assert(a && f);
  assert(ant_posx(a) >= 0); assert(ant_posx(a) < field_xmax(f));
  assert(ant_posy(a) >= 0); assert(ant_posy(a) < field_ymax(f));

  /* Choose the best cap */
  int direction = ant_best_direction(a, f);
  switch(direction) {
  case NORTH:
    ant_movey(a, -1);
    break;
  case EAST:
    ant_movex(a, 1);
    break;
  case SOUTH:
    ant_movey(a, 1);
    break;
  case WEST:
    ant_movex(a, -1);
    break;
  case NORTH_EAST:
    ant_movey(a, -1);
    ant_movex(a, 1);
    break;
  case SOUTH_EAST:
    ant_movey(a, 1);
    ant_movex(a, 1);
    break;
  case SOUTH_WEST:
    ant_movey(a, 1);
    ant_movex(a, -1);
    break;
  case NORTH_WEST:
    ant_movey(a, -1);
    ant_movex(a, -1);
    break;
  default:
    break;
  }

  /* Update orientation */
  ant_updt_direction(a, direction);
  /* Put pheromenes */
  if(ant_is_loaded(a))
    ant_put_down_pheromone(a, f);
  /* Work */
  ant_can_collect(a, f);
  ant_can_deposit(a, f);

  return 1;
}




