/*
 * ANTs - Fourmis informatiques
 * Copyright (C) 2022 MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

#define _FIELD_C

#include <stdio.h>
#include <stdlib.h>

#include <math.h>

#include <assert.h>

#include "field.h"


/**
 * Display field.
 */
int field_display(Field f) {
  /* Assertion */
  assert(f);

  /* Display */
  if(field_xmax(f) > 100 || field_ymax(f) > 100) return 0;  /* too big! */
  //setbuf(stdout, NULL);  /* disable buffering on stdout */
  for(int j=0; j<field_ymax(f); j++) {
    for(int i=0; i<field_xmax(f); i++) {
      switch(field_is_busy(f, i, j)) {
      case NONE:
        double p = field_taux_pheromone(f, i, j) * MAX_PHEROMONE;
        if(MAX_PHEROMONE < 100)
          if(p > 0)
            printf("%2.0f", p);
          else
            printf("  ");
        else {
          if(p >= (MAX_PHEROMONE*4/5))
            printf(" +");
          else if(p >= (MAX_PHEROMONE*1/2))
            printf(" ~");
          else if(p > 0)
            printf(" -");
          else
            printf("  ");
        }
        break;
      case FOOD:
        printf("ff"); break;
      case OBSTACLE:
        printf("xx"); break;
      default:
        printf(" ?"); break;
      }
    }
    printf("\n");
  }
  fflush(stdout);  /* will now print everything in the stdout buffer */

  return 0;
}
