/*
 * ANTs - Fourmis informatiques
 * Copyright (C) 2022 MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

/**
 * Definition of the Field struct., and public interface.
 */

#ifndef _FIELD_H
#define _FIELD_H

/* Objects types */
#define NONE 0
#define FOOD 1
#define OBSTACLE 2

/* Max pheromone rate */
#define MAX_PHEROMONE 99
#define PHEROMONE_EVAPORATION 500  /* step */

/* The field */
typedef struct Field *Field;


/**
 * Constructor.
 * @param x  The width of the Field.
 * @param y  The height of the Field.
 * @return a new Field.
 */
Field field_new(int x, int y);

/**
 * Destructor.
 * @param f  The Field.
 * @return 1.
 */
int field_destroy(Field f);


/**
 * SizeX (begin at 0).
 * @param f  The Field.
 * @return the width.
 */
int field_xmax(Field f);

/**
 * SizeY (begin at 0).
 * @param f  The Field.
 * @return the height.
 */
int field_ymax(Field f);


/**
 * Field is empty?
 * @param f  The Field.
 * @return 0 if some object left, else 0.
 */
int field_is_empty(Field f);

/**
 * Object encounterred? Or, pos (x,y) is busy?
 * @param f  The Field.
 * @param x  The pos X.
 * @param y  The pos Y.
 * @return the object or NONE.
 */
int field_is_busy(Field f, int x, int y);

/**
 * Update field (object) in pos (x, y).
 * @param f  The Field.
 * @param x  The pos X.
 * @param y  The pos Y.
 * @return the object.
 */
int field_put_down(Field f, int x, int y, int o);


/**
 * Return pheromone rate on pos (x,y).
 * @param f  The Field.
 * @param x  The pos X.
 * @param y  The pos Y.
 * @return the pheromone rate. 
 */
double field_taux_pheromone(Field f, int x, int y);

/**
 * Put pheromones at pos (x, y).
 * @param f  The Field.
 * @param x  The pos X.
 * @param y  The pos Y.
 * @return the new pheromone level.
 */
int field_incr_pheromone(Field f, int x, int y);

/**
 * Pheromones are volatilizable...
 * @param f  The Field.
 * @return 1.
 */
int field_pheromone_evaporation(Field f);


/**
 * Display field.
 * @param f  The Field.
 * @return 0.
 */
int field_display(Field f);


#endif /* _FIELD_H */
