/*
 * ANTs - Fourmis informatiques
 * Copyright (C) 2000 MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

/**
 * Definition of the Nest struct.,
 * and public interface.
 */

#ifndef _NEST_H
#define _NEST_H

/* Objects types */
#define NONE 0
#define EGG 1
#define LARVA 2
#define COCOON 3

/* The nest */
typedef struct Nest *Nest;


/**
 * Constructor.
 * @param x  The width of the Nest.
 * @param y  The height of the Nest.
 * @return a new Nest.
 */
Nest nest_new(int x, int y);

/**
 * Destructor.
 * @param n  The Nest.
 * @return 1.
 */
int nest_destroy(Nest n);


/**
 * SizeX (begin at 0).
 * @param n  The Nest.
 * @return the width.
 */
int nest_xmax(Nest n);

/**
 * SizeY (begin at 0).
 * @param n  The Nest.
 * @return the height.
 */
int nest_ymax(Nest n);


/**
 * Object encounterred? Or, pos (x,y) is busy?
 * @param n  The Nest.
 * @param x  The pos X.
 * @param y  The pos Y.
 * @return the object or NONE.
 */
int nest_is_busy(Nest n, int x, int y);

/**
 * Update nest (object) in pos (x, y).
 * @param n  The Nest.
 * @param x  The pos X.
 * @param y  The pos Y.
 * @return the object.
 */
int nest_put_down(Nest n, int x, int y, int o);


/**
 * Display nest.
 * @param n  The Nest.
 * @return 0.
 */
int nest_display(Nest n);


#endif /* _NEST_H */
