/* 
 * Stack abstract data type - Last In First Out.
 * Copyright (C) 2003 MiKael NAVARRO
 *
 * This library is free software; you can redistribute it and/or
 * modify it under the terms of the GNU Library General Public
 * License as published by the Free Software Foundation; either
 * version 2 of the License, or (at your option) any later version.
 *
 * This library is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
 * Library General Public License for more details.
 *
 * You should have received a copy of the GNU Library General Public
 * License along with this library; if not, write to the
 * Free Software Foundation, Inc., 59 Temple Place - Suite 330,
 * Boston, MA  02111-1307, USA.
 */

/**
 * Definition of the Stack struct., and public interface.
 */

#ifndef _STACK_H
#define _STACK_H

/* Abstract data type */
typedef struct Stack *Stack;


/**
 * Constructor.
 * @return a new Stack.
 */
Stack stack_new(void);

/**
 * Destructor.
 * @param stack  The Stack.
 * @return 1.
 */
int stack_destroy(Stack stack);


/**
 * Push n.
 * @param stack  The Stack.
 * @param n  The number.
 * @return the number pushed.
 */
int stack_push(Stack stack, double n);

/**
 * Pop.
 * @param stack  The Stack.
 * @return the number poped or 0.
 */
double stack_pop(Stack stack);


/**
 * Top.
 * @param stack  The Stack.
 * @return the number on top.
 */
double stack_top(Stack stack);

/**
 * Head idx.
 * @param stack  The Stack.
 * @return the top index.
 */
int stack_top_index(Stack stack);


/**
 * Stack is full? 0|1
 * @param stack  The Stack.
 * @return 1 if Stack is full, else 0.
 */
int stack_is_full(Stack stack);

/**
 * Stack is empty? 0|1
 * @param stack  The Stack.
 * @return 1 if Stack is empty, else 0.
 */
int stack_is_empty(Stack stack);


/**
 * Get elem (pos=i). 
 * @param stack  The Stack.
 * @return the number at index i, or 0.
 */
double stack_elem(Stack stack, int i);


/**
 * Display stack.
 * @param stack  The Stack.
 * @return 0.
 */
int stack_aff(Stack stack);


#endif /* _STACK_H */
